#!/usr/bin/python
import xmlrpclib
import json

from .groups import Groups

class HWtypes:
    """ manage configuration for hardware types """
    def __init__(self):
        self.groups = Groups() # hwtype groups
        self.hwtypes = {}      # group formulas
        self.client = None
        self.key = None

    def add(self, hw_name, desc, saltboot):
        """ add new entry for hwtype group with corresponding saltboot formula """
        self.hwtypes[hw_name] = {
            'saltboot': saltboot
        }
        self.groups.add("HWTYPE:" + hw_name, desc)

    def connect(self, client, key):
        """ connect to API """
        self.client = client
        self.key = key

    def disconnect(self):
        """ disconnect from API """
        self.client = None

    def upload(self):
        """ upload grups and group formulas to API """
        self.groups.upload(self.client, self.key)

        for hw_name in self.hwtypes:
            group = "HWTYPE:" + hw_name
            group_id = self.groups.get_id(group)
            enabled_formulas = set(self.client.formula.getFormulasByGroupId(self.key, group_id))
            enabled_formulas.update(self.hwtypes[hw_name].keys())
            enabled_formulas = list(enabled_formulas)

            self.client.formula.setFormulasOfGroup(self.key, group_id, enabled_formulas)
            for formula in self.hwtypes[hw_name]:
                self.client.formula.setGroupFormulaData(self.key, group_id, formula, self.hwtypes[hw_name][formula])

    def download(self):
        """ download hwtype groups and group formulas from API """

        self.groups.download(self.client, self.key)
        groups = self.groups.get_existing()

        for group in groups:
            group_id = self.groups.get_id(group)
            if group.startswith("HWTYPE:"):
                hw_name = group[len("HWTYPE:"):]
                saltboot = self.client.formula.getGroupFormulaData(self.key, group_id, "saltboot")
                self.add(hw_name, self.groups.get_attr(group, "description"), saltboot)

    def from_yaml(self, yaml):
        """ add group entries and corresponding saltboot formulas defined in yaml structure """
        for hw_name in yaml:
            self.add(hw_name, yaml[hw_name].get("description", "Hardware group for {}".format(hw_name)), yaml[hw_name].get("saltboot", {}))

    def to_yaml(self):
        """ download hwtype groups and group formulas from API and return them as yaml structure """
        self.download()

        yaml = {}

        for hw_name in self.hwtypes:
            group = "HWTYPE:" + hw_name
            yaml[hw_name] = {
                'description': self.groups.get_attr(group, "description"),
                'saltboot': self.hwtypes[hw_name].get('saltboot', {})
            }
        return yaml
