import logging

log = logging.getLogger(__name__)


class Terminal:
    """ handle mapping between high-level configuration and terminal entry"""

    def __init__(self, branch_prefix, hostname=None, hwAddress=None, system_id=None):
        self.branch_prefix = branch_prefix
        self.hostname = hostname
        self.hwAddress = hwAddress
        self.ip = None
        self.saltboot = None
        self.hwtype = None
        self.system_id = system_id

    def connect(self, client, key):
        """ connect to API """
        self.client = client
        self.key = key

    def upload(self):
        if not self.system_id:
            self.client.system.createSystemProfile(self.key, self.branch_prefix + '.' + self.hostname, {'hwAddress': self.hwAddress})

            ids = self.client.system.getId(self.key, self.branch_prefix + '.' + self.hostname)
            if not isinstance(ids, list) or len(ids) == 0:
                log.error("Could not create '%s'", self.branch_prefix + '.' + self.hostname)
                return False

            self.system_id = ids[0]['id']

        self.client.systemgroup.addOrRemoveSystems(self.key, self.branch_prefix, [self.system_id], True)
        if self.hwtype is not None:
            self.client.systemgroup.addOrRemoveSystems(self.key, 'HWTYPE:' + self.hwtype, [self.system_id], True)

        if self.saltboot:
            self.client.formula.setFormulasOfServer(self.key, self.system_id, ['saltboot'])
            self.client.formula.setSystemFormulaData(self.key, self.system_id, 'saltboot', self.saltboot)
        else:
            self.client.formula.setFormulasOfServer(self.key, self.system_id, [])


    def download(self):
        try:
            self.hostname = self.client.system.getNetwork(self.key, self.system_id)['hostname']
            if not self.hostname or self.hostname == '(none)':
                raise ValueError()
        except:
            self.hostname = self.client.system.getName(self.key, self.system_id)['name']

        if self.hostname.startswith(self.branch_prefix + '.'):
            self.hostname = self.hostname[len(self.branch_prefix) + 1 : ]
        dotpos = self.hostname.find('.')
        if dotpos != -1:
            self.hostname = self.hostname[:dotpos]

        net_devices = self.client.system.getNetworkDevices(self.key, self.system_id)
        for nic in net_devices:
            if nic.get('interface') != 'lo' and nic.get('hardware_address'):
                self.hwAddress = nic.get('hardware_address')
                self.ip = nic.get('ip')
                break

        enabled_formulas = set(self.client.formula.getFormulasByServerId(self.key, self.system_id))
        if 'saltboot' in enabled_formulas:
            self.saltboot = self.client.formula.getSystemFormulaData(self.key, self.system_id, 'saltboot')
        else:
            self.saltboot = None

        groups = self.client.system.listGroups(self.key, self.system_id)
        for g in groups:
            if g.get('system_group_name', '').startswith('HWTYPE:') and g.get('subscribed'):
                self.hwtype = g['system_group_name'][len('HWTYPE:') : ]
                break


    def from_yaml(self, yaml):
        """ configure terminal from yaml structure """
        self.hwAddress = yaml.get('hwAddress')
        self.ip = yaml.get('IP')
        self.saltboot = yaml.get('saltboot')
        self.hwtype = yaml.get('hwtype')

    def to_yaml(self):
        ret = { self.hostname: {'hwAddress': self.hwAddress }}
        if self.ip:
            ret[self.hostname]['IP'] = self.ip
        if self.saltboot:
            ret[self.hostname]['saltboot'] = self.saltboot
        if self.hwtype:
            ret[self.hostname]['hwtype'] = self.hwtype
        return ret
