#!/usr/bin/python
import yaml
import logging
from .branch import Branch, list_branch_servers
from .hwtypes import HWtypes

log = logging.getLogger(__name__)

class YAMLstructure:
    """ load and save SUSE Manager for Retail configuration as yaml file """
    def __init__(self):
        self.yaml = {
            'branches': {},
            'hwtypes': {}
        }

    def loadFile(self, name):
        """ load configuration from yaml file """
        with open(name, 'r') as infile:
            self.yaml = yaml.safe_load(infile)

    def saveFile(self, name):
        """ save onfiguration as yaml file """
        with open(name, 'w') as outfile:
            yaml.safe_dump(self.yaml, outfile, default_flow_style=False)

    def upload(self, client, key):
        """ upload configuration to SUSE Manager API"""

        # hwtype groups must be created first, terminal entries may rely on them
        hwtypes = HWtypes()
        hwtypes.connect(client, key)
        hwtypes.from_yaml(self.yaml.get('hwtypes', {}))

        branches = []
        for branch_server_id in self.yaml.get('branches', {}):
            branch_yaml = self.yaml.get('branches', {}).get(branch_server_id, {})
            branch = Branch(branch_server_id, branch_yaml.get('hwAddress'))
            if branch.connect(client, key):
                branch.from_yaml(branch_yaml)
                branches.append(branch)
            else:
                log.warning("'%s' does not exist, skipping.", branch_server_id)

        # hwtype groups must be created first, terminal entries may rely on them
        hwtypes.upload()
        hwtypes.disconnect()
        for branch in branches:
                branch.upload()
                branch.disconnect()


    def download(self, client, key):
        """ download configuration from SUSE Manager API"""
        for branch_server_id in list_branch_servers(client, key):
            branch = Branch(branch_server_id)
            branch.connect(client, key)
            self.yaml['branches'][branch_server_id] = branch.to_yaml()
            branch.disconnect()

        hwtypes = HWtypes()
        hwtypes.connect(client, key)
        self.yaml['hwtypes'] = hwtypes.to_yaml()
        hwtypes.disconnect()
